import pandas as pd
import requests
from lxml import etree

def process_one_url(url:str) -> list:
    html = requests.get(url).text
    li = list(etree.HTML(html).xpath("//p//text()"))
    li = list(filter(lambda x: len(x.split(" ")) > 50, li))
    li = [x[1:].replace('\n', ' ') for x in li]
    return li

def query_abstracts_arxiv(urls: list) ->pd.DataFrame:
    ret = []
    for url in urls:
        ret.extend(process_one_url(url))
    df = pd.DataFrame(ret)
    return df




import requests
import pandas as pd
from lxml import etree

def exchange_rate_cn() -> pd.DataFrame:
    headers = {
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/95.0.4638.54 Safari/537.36',
    }
    html = requests.get('https://www.waihui321.com/', headers=headers).text
    tmp = etree.HTML(html).xpath('//table[@class="boc"]//tr//text()')
    rates = []

    for i in range(0, len(tmp) - 7, 7):
        rates.append(tmp[i:i + 7])

    rates[0][1] = '代码'

    return pd.DataFrame(rates)

if __name__ == '__main__':
    print(exchange_rate_cn())



import sys
import requests
from typing import List, Union
import xmltodict
import json
class BaseQuery:
    # class atributes

    PACKAGE_ROOT = '.'



    HEADERS = {
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/95.0.4638.54 Safari/537.36',
    }
    @classmethod
    def __build_params__(cls, *args,**kwargs):
        raise NotImplemented

    @classmethod
    def __query__(cls, url:str, params:dict):
        '''

        :param url:
        :param params:
        :return: list of dicts, each dict contains info of a paper
        '''
        response = requests.get(url=url,params=params,headers=cls.HEADERS)
        return response

class PaperWithCodeQuery(BaseQuery):
    ENDPOINT = 'https://paperswithcode.com/api/v1/papers/'

    @classmethod
    def __build_params__(cls,query: str,
        page = 1,
        items_per_page = 50
        ):

        return {
            'page': page,
            'items_per_page': items_per_page,
            'title': query
        }

    @classmethod
    def __query__(cls, url: str, params: dict):
        return super().__query__(url, params)

    @classmethod
    def query(cls, query: str, page = 1, items_per_page = 50):
        params =  cls.__build_params__(query,page,items_per_page)
        return cls.__query__(cls.ENDPOINT, params).json()['results']


class ArxivQuery(BaseQuery):
    ENDPOINT = 'http://export.arxiv.org/api/query?'
    '''
    an example request:
    'http://export.arxiv.org/api/query?search_query=bert&max_results=1'
    '''
    @classmethod
    def __build_params__(cls,
                         query: str,
                         id_list: str = '',
                         start: int = 0,
                         max_results: int = 50
                         ):

        return {
            'search_query': query,
            'id_list': id_list,
            'start': start,
            'max_results': max_results
        }

    @classmethod
    def query(cls,
              query: str,
              id_list: str= '',
              start: int = 0,
              max_results: int = 50
              ) -> Union[dict, List[dict]]:
        params = cls.__build_params__(query,id_list,start,max_results)
        xml = cls.__query__(cls.ENDPOINT,params).content
        tmp = xmltodict.parse(xml)
        return tmp['feed']['entry']


class IEEEQuery(BaseQuery):
    '''
    an example query url:
    https://ieeexploreapi.ieee.org/api/v1/search/articles?parameter&apikey={API_KEY}&querytext=BERT&max_records=200&start_year=2019&end_year=2021
    '''

    '''
    README:
    Please make sure a .config.json file is stored under the package root
    A template would be like this:
    {
        "api_key": "my api key"
    }
    '''
    ENDPOINT = 'https://ieeexploreapi.ieee.org/api/v1/search/articles?parameter&apikey='

    @classmethod
    def __build_params__(cls, query: str,start_year: int ,end_year: int,num_papers: int = 200 ):
        num_papers = min(num_papers,200)
        end_year = max(start_year,end_year)
        return {
            'querytext': query,
            'max_records': num_papers,
            'start_year': start_year,
            'end_year': end_year
        }

    @classmethod
    def query(cls,
              query: str,
              start_year: int ,
              end_year: int,
              num_papers: int = 200
              ) -> Union[dict, List[dict]]:
        try:
            with open(f"{cls.PACKAGE_ROOT}/.config.json", 'r', encoding='UTF-8') as f:
                api_key = json.load(f)['api_key']
        except Exception:
            print('Please first call "IEEEQuery.__setup_api_key__(<your api key>)" to store the api key first.',file=sys.stderr)
            sys.exit()

        endpoint = cls.ENDPOINT + api_key + '&'
        params = cls.__build_params__(query,start_year,end_year,num_papers)
        papers = cls.__query__(endpoint,params).json()['articles']
        return papers

    @classmethod
    def __setup_api_key__(cls, api_key:str):
        config = {
            'api_key': api_key
        }

        jsObj = json.dumps(config)
        with open(f'{cls.PACKAGE_ROOT}/.config.json', 'w') as f:
            f.write(jsObj)











import asyncio
from typing import List
import time
import json
import os
from lxml import etree
from tqdm import tqdm
import requests
import aiohttp
class AirQualityQuery:
    KEY = '550b1b06-665d-41f5-b606-c26d06646a20'
    ENDPOINT = '''http://api.airvisual.com/v2/'''
    PATH = '.CITIES.json'
    HEADERS = {
  'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/95.0.4638.54 Safari/537.36',
}

    if os.path.exists(PATH):
        with open(PATH, 'r', encoding='UTF-8') as f:
            CITIES = json.load(f)
    else:
        CITIES = dict()

    @classmethod
    async def tasks(cls, tasks: tuple):
        await asyncio.gather(*tasks)

    @classmethod
    def __get__provinces__(cls,country: str) -> List[str]:
        url = cls.ENDPOINT + 'states?'
        tmp = requests.get(url,dict(
            country=country,
            key=cls.KEY
        )).json()

        response_state = tmp['status']
        while response_state != 'success':
            time.sleep(10)
            tmp = requests.get(url, dict(
                country=country,
                key=cls.KEY
            )).json()
            response_state = tmp['status']
        provinces = [x['state'] for x in tmp['data']]
        return provinces

    @classmethod
    def __get__cities_in_province__(cls,province: str, country: str) -> List[str]:
        url = cls.ENDPOINT + 'cities?'
        tmp = requests.get(url,dict(
            state = province,
            key=cls.KEY,
            country = country
        )).json()

        response_state = tmp['status']
        while response_state != 'success':
            time.sleep(10)
            tmp = requests.get(url, dict(
                state=province,
                key=cls.KEY,
                country=country
            )).json()
            response_state = tmp['status']

        cities = [x['city'] for x in tmp['data']]
        return cities

    @classmethod
    def __get_cities_in_country__(cls,country:str) -> List[str]:
        if country not in cls.CITIES:
            cls.CITIES[country] = dict()
            provinces = cls.__get__provinces__(country)  # [str]
            for i in tqdm(range(len(provinces))):
                province = provinces[i]
                cities = cls.__get__cities_in_province__(province, country)
                cls.CITIES[country][province] = cities

            with open(cls.PATH,'w') as f:
                f.write(json.dumps(cls.CITIES))

        return cls.CITIES[country]

    @classmethod
    def air_quality_by_country(cls, country: str):
        if country not in cls.CITIES:
            data = cls.__get_cities_in_country__(country)
        else:
            data = cls.CITIES[country]

        aqi_values = []

        for prov in data.keys():
            # for city in cities:
            #     html = requests.get(f'''{url}/{prov.lower()}/{city.lower()}''', headers=cls.HEADERS).text
            #     try:
            #         aqi = int(etree.HTML(html).xpath('//p[@class="aqi-value__value"]//text()')[0])
            #     except:
            #         continue
            #     aqi_values.append((city,prov, aqi))
            #     if return_frequency and len(aqi_values) % return_frequency ==0:
            #         aqi_values.sort(key=lambda x: x[2])
            #         yield aqi_values
            aqi_values += cls.air_quality_by_province_country(country,prov)
            aqi_values.sort(key=lambda x: x[2])
            yield aqi_values


        aqi_mean = sum([x[2] for x in aqi_values]) / len(aqi_values)
        aqi_values.append(('mean', country, aqi_mean))
        yield aqi_values

    @classmethod
    def air_quality_by_province_country(cls,country:str,province_:str):
        province = province_[0].upper()
        province += province_[1:]

        if country not in cls.CITIES:
            data = cls.__get_cities_in_country__(country)
        else:
            data = cls.CITIES[country]
        if province not in data.keys():
            raise RuntimeError('There is no such province in this country.')

        cities = data[province] # [str]
        url = f'''https://www.iqair.com/{country.lower()}/{province.lower()}'''
        aqi_values = []

        async def task(city:str):
            # html = requests.get(f'''{url}/{city.lower()}''', headers=cls.HEADERS).text
            async with aiohttp.ClientSession() as session:
                async with session.get(f'''{url}/{city.lower()}''', headers=cls.HEADERS) as response:
                    html = await response.text()
            try:
                aqi = int(etree.HTML(html).xpath('//p[@class="aqi-value__value"]//text()')[0])
            except:
                return
            aqi_values.append((city, province, aqi))
        asyncio.run(cls.tasks(tuple(task(city) for city in cities)))

        if len(aqi_values) == 0:
            return aqi_values
        aqi_values.sort(key=lambda x: x[2])
        aqi_mean = sum([x[2] for x in aqi_values])/len(aqi_values)
        aqi_values.append(('mean',province,aqi_mean))
        return aqi_values


if __name__ == '__main__':
    # print(AirQualityQuery.__get__provinces__('china'))
    generator = AirQualityQuery.air_quality_by_country('china')
    for i in generator:
        print(i)
        print()
    # print(AirQualityQuery.air_quality_by_province_country('china','fujian'))
    # print(AirQualityQuery.air_quality_by_country('china'))
